@@ -370,7 +370,9 @@ server <- function(input, output, session, ...) {
             data <- httr::content(resp)$timeserie
             json <- jsonlite::toJSON(data, auto_unbox=T, null = "null")
             res_df <- as.data.frame(jsonlite::fromJSON(json, simplifyDataFrame=T))
-            res_df$source <- df[["source__source_name"]][[i]]
+            if (nrow(res_df) > 0) {
+              res_df$source <- df[["source__source_name"]][[i]]
+            }
             res_df
          } else {
            stop("Error while obatining data series")
@@ -379,54 +381,57 @@ server <- function(input, output, session, ...) {
       progress_set(value = 0.9, message = "Joining results", rep)
       ts_df <- jsonlite::rbind_pages(results)
       progress_set(value = 0.9, message = "Plotting time series", rep)
-
       title <- ""#paste(sapply(strsplit(unique(ts_df$indicator), "_|-| "), function(t) paste(sapply(t, function(w) paste(toupper(substr(w, 1, 1)), tolower(substr(w, 2, nchar(w))), sep = "")), collapse = " ")), collase = ", ")
       singles = list()
       details = list()
-      ts_df$legend <- sapply(ts_df$key, function(v) "")
-      # creating labels and title
-      for(col in c(keys)) {
-        label_col <- paste(col, "label", sep = "_")
-        values <- df[[col]]
-        if(label_col %in% colnames(df)) {
-          labels <- df[[label_col]]
-        }
-        else
-          labels <- values
-        values <- setNames(values, labels)
-        values <- values[sort(unique(names(values)), na.last = TRUE)]
-        labels <- setNames(
-          sapply(strsplit(names(values), "_|-| "), function(t) paste(sapply(t, function(w) paste(toupper(substr(w, 1, 1)), tolower(substr(w, 2, nchar(w))), sep = "")), collapse = " ")),
-          values
-        )
-        df_labels <- sapply(1:nrow(ts_df), function(i) {
-          json <- ts_df$key[[i]]
-          keys <-  jsonlite::fromJSON(json)
-          if(col == "indicator")
-            label = ts_df$indicator[[i]]
-          else if(col %in% names(keys)) {
-            label = labels[[as.character(keys[[col]])]]
+      if(nrow(ts_df)>0) {
+        ts_df$legend <- sapply(ts_df$key, function(v) "")
+        # creating labels and title
+        for(col in c(keys)) {
+          label_col <- paste(col, "label", sep = "_")
+          values <- df[[col]]
+          if(label_col %in% colnames(df)) {
+            labels <- df[[label_col]]
           }
-          else 
-            label = NA
-          label
-        })
-        if(col == "indicator")
-          title <- paste(unique(labels), collapse = ", ")
-        if(length(values)>0 && !all(is.na(values)))
-          details[[col]] = sort(unique(labels))
-        if(length(unique(labels))==1 && !is.na(values)) {
-          #if(unique(labels) != "All" && col != "period_type")
-          #  title = paste(title, ", ", unique(labels), sep = "") 
-          singles[[col]] = unique(labels) 
-        } else if(length(unique(labels))>1) {
-          ts_df$legend = sapply(1:nrow(ts_df), function(i) {
-            leg = ts_df[["legend"]][[i]]
-            if(nchar(leg)>0)
-              leg = paste(leg, ", ")
-            leg = paste(leg, df_labels[[i]])
-            leg
+          else
+            labels <- values
+          values <- setNames(values, labels)
+          values <- values[sort(unique(names(values)), na.last = TRUE)]
+          labels <- setNames(
+            sapply(strsplit(names(values), "_|-| "), function(t) paste(sapply(t, function(w) paste(toupper(substr(w, 1, 1)), tolower(substr(w, 2, nchar(w))), sep = "")), collapse = " ")),
+            values
+          )
+          df_labels <- sapply(1:nrow(ts_df), function(i) {
+            json <- ts_df$key[[i]]
+            if (length(json) != 0) {
+              keys <-  jsonlite::fromJSON(json)
+              if(col == "indicator")
+                label = ts_df$indicator[[i]]
+              else if(col %in% names(keys)) {
+                label = labels[[as.character(keys[[col]])]]
+              }
+              else 
+                label = NA
+              label
+            }
           })
+          if(col == "indicator")
+            title <- paste(unique(labels), collapse = ", ")
+          if(length(values)>0 && !all(is.na(values)))
+            details[[col]] = sort(unique(labels))
+          if(length(unique(labels))==1 && !is.na(values)) {
+            #if(unique(labels) != "All" && col != "period_type")
+            #  title = paste(title, ", ", unique(labels), sep = "") 
+            singles[[col]] = unique(labels) 
+          } else if(length(unique(labels))>1) {
+            ts_df$legend = sapply(1:nrow(ts_df), function(i) {
+              leg = ts_df[["legend"]][[i]]
+              if(nchar(leg)>0)
+                leg = paste(leg, ", ")
+              leg = paste(leg, df_labels[[i]])
+              leg
+            })
+          }
         }
       }
       output$timeseries_title <- shiny::renderText({
@@ -466,18 +471,20 @@ server <- function(input, output, session, ...) {
          # Validate if minimal requirements for rendering are met 
          progress_set(value = 0.15, message = "Generating line chart", rep)
          ts_df$date <- as.Date(ts_df$date)
-         chart <- plot_timeseries(ts_df, title = title, scale = input$timeseries_scale, period = input$timeseries_period)
-         
-         # Setting size based on container size
-         height <- session$clientData$output_line_chart_height
-         width <- session$clientData$output_line_chart_width
-	       
-         # returning empty chart if no data is found on chart
-         chart_not_empty(chart)
+         if (nrow(ts_df) > 0) {
+          chart <- plot_timeseries(ts_df, title = title, scale = input$timeseries_scale, period = input$timeseries_period)
          
+          # Setting size based on container size
+          height <- session$clientData$output_line_chart_height
+          width <- session$clientData$output_line_chart_width
+          
+          # returning empty chart if no data is found on chart
+          chart_not_empty(chart)
+          
 
-         # transforming chart on plotly
-         gg <- plotly::ggplotly(chart, height = height, width = width, tooltip = c("label")) %>% plotly::config(displayModeBar = FALSE) 
+          # transforming chart on plotly
+          gg <- plotly::ggplotly(chart, height = height, width = width, tooltip = c("label")) %>% plotly::config(displayModeBar = FALSE) 
+         }
       }) 
       progress_close(env = rep)
     }
@@ -808,7 +815,7 @@ plot_timeseries <- function(df, title, scale = "std", period = "12") {
   } else 
     numeric(0)
   # Calculating breaks of y axis
-  y_breaks <- unique(floor(pretty(seq(0, (max(df$value, na.rm=T) + 1) * 1.1))))
+  #y_breaks <- unique(floor(pretty(seq(0, (max(df$value, na.rm=T) + 1) * 1.1))))
   # plotting
   labels <- unique(df$legend)
   fig_line <- ggplot2::ggplot(df, ggplot2::aes(x = .data$date, y = .data$y, color = .data$legend, label = .data$Details)) +
