@@ -1,7 +1,7 @@
 import pandas as pd
 
 def df_transform(df):
-  filtered_df = df.groupby(['day', 'destination'])['callsign'].count()
+  filtered_df = df.groupby(['day', 'destination', 'origin'])['callsign'].count()
   filtered_df = filtered_df.reset_index()
   filtered_df['line_number'] = range(1, len(filtered_df)+1) 
   return filtered_df.rename(columns={"callsign": "flights"})
\ No newline at end of file
