@@ -17,7 +17,7 @@ l = logging.getLogger("pandem.pipeline")
 
 class Pipeline(worker.Worker):
     __metaclass__ = ABCMeta  
-    def __init__(self, name, orchestrator_ref, settings, retry_failed = False, restart_job = 0, retry_active = True): 
+    def __init__(self, name, orchestrator_ref, settings, retry_failed = False, restart_job = -1, retry_active = True): 
         super().__init__(name = name, orchestrator_ref = orchestrator_ref, settings = settings)
         self.retry_failed = retry_failed
         self.retry_active = retry_active
@@ -41,6 +41,7 @@ class Pipeline(worker.Worker):
 
         self.job_steps = defaultdict(dict)
         self.decompressed_files = defaultdict(list)
+        self.job_precalstamp= {}
         self.job_precalstep = {}
         self.job_df = defaultdict(dict)
         self.job_tuples = defaultdict(dict)
@@ -56,7 +57,7 @@ class Pipeline(worker.Worker):
 
         self.job_dicos = [
           self.decompressed_files, self.job_df, self.job_tuples, self.job_stdtuples, self.job_aggrtuples, 
-          self.job_precalinds, self.job_indicators, self.pending_count, self.pending_total, self.job_issues
+          self.job_precalinds, self.job_indicators, self.pending_count, self.pending_total, self.job_issues, self.job_precalstamp, self.job_precalstep
         ]
 
         self.last_step = "ended"
@@ -66,7 +67,7 @@ class Pipeline(worker.Worker):
                                                  ((x['status'] == 'in progress' and self.retry_active) or (x['status'] == 'failed' and self.retry_failed)) 
                                                    and x['step'] != 'submitted_started' 
                                                    and x['step'] != self.last_step
-                                                   and self.restart_job == 0
+                                                   and self.restart_job == -1
                                              ) or x["id"] == self.restart_job
                                          ).get()
         if jobs is not None :
@@ -219,39 +220,52 @@ class Pipeline(worker.Worker):
         
         # Jobs after precalculate ends
         jobs = self.job_steps['precalculate_ended'].copy().values()
-        ivar = 0
+        ipart = 0
         done = False
         while not done: 
           done = True
           for job in jobs:
-            var_list = [*self.job_aggrtuples[job["id"]].keys()]
-            if ivar == 0:
-              self.pending_count[job["id"]] = len(var_list)
-              self.pending_total[job["id"]] = len(var_list)
+            part_list = [*self.job_aggrtuples[job["id"]].keys()]
+            if ipart == 0:
+              self.pending_count[job["id"]] = len(part_list)
+              self.pending_total[job["id"]] = len(part_list)
               self.update_job_step(job, 'publish_started', 0.9)
-            if ivar < len(var_list):
+            if ipart < len(part_list):
               done = False
-              tuples = self.job_aggrtuples[job["id"]][var_list[ivar]]
+              tuples = self.job_aggrtuples[job["id"]][part_list[ipart]]
               self.send_to_publish(tuples, job)
-            if ivar == len(var_list):
+            if ipart == len(part_list):
               # aggregated tuples are not necessary anymore
               self.job_aggrtuples.pop(job["id"])
-          ivar = ivar + 1
+          ipart = ipart + 1
 
 
         # Jobs after calculate ends
-        for job in self.job_steps['calculate_ended'].copy().values():
-            indicators_tuples = self.job_indicators[job["id"]]
-            self.update_job_step(job, 'publish_started', 0.9)
-            self.pending_count[job["id"]] = 1
-            self.send_to_publish(indicators_tuples, job)
+        jobs = self.job_steps['calculate_ended'].copy().values()
+        ipart = 0
+        done = False
+        while not done:
+          done = True
+          for job in jobs:
+            part_list = [*self.job_indicators[job["id"]].keys()]
+            if ipart == 0:
+              self.pending_count[job["id"]] = len(part_list)
+              self.pending_total[job["id"]] = len(part_list)
+              self.update_job_step(job, 'publish_started', 0.9)
+            if ipart < len(part_list):
+              done = False
+              indicators_tuples = self.job_indicators[job["id"]][part_list[ipart]]
+              self.send_to_publish(indicators_tuples, job)
+            if ipart == len(part_list):
+              pass
+          ipart = ipart + 1
 
         # Jobs after publish indicator ended
         for job in self.job_steps['publish_ended'].copy().values():
             calc_step = self.job_precalstep[job['id']] + 1
             self.job_precalstep[job['id']] = calc_step
             # recreating the indicators to calculate for the current step
-            ind_in_step = {k:v for k,v in self.job_precalinds[job['id']].items() if v["step"] == calc_step}
+            ind_in_step = [(k,v) for k,v in self.job_precalinds[job['id']] if v["step"] == calc_step]
             if len(ind_in_step) > 0:
               self.update_job_step(job, 'calculate_started', 0.8)
               self.send_to_calculate(ind_in_step, job)
@@ -333,8 +347,11 @@ class Pipeline(worker.Worker):
         self.pending_count[job["id"]] = self.pending_count[job["id"]] - 1
         self.job_tuples[job["id"]][path] = tuples
         self.job_issues[job["id"]].extend(issues)
-        
-        progress =  (1.0/10) * len(self.job_tuples[job["id"]]) / (self.pending_count[job["id"]] + len(self.job_tuples[job["id"]]))
+       
+        if (self.pending_count[job["id"]] + len(self.job_tuples[job["id"]])) == 0:
+          progress = 0.0
+        else: 
+          progress =  (1.0/10) * len(self.job_tuples[job["id"]]) / (self.pending_count[job["id"]] + len(self.job_tuples[job["id"]]))
         
         if self.pending_count[job["id"]] == 0:
             self.write_issues(self.job_issues[job["id"]]) 
@@ -362,8 +379,10 @@ class Pipeline(worker.Worker):
         self._storage_proxy.delete_db("issue", lambda i: int(i["job_id"]) == int(job['id']) and i['issue_type'] == "ref-not-found").get()
         
         self.job_issues[job["id"]].extend(issues)
-        
-        progress =  (1.0/10) * len(self.job_stdtuples[job["id"]]) / (self.pending_count[job["id"]] + len(self.job_stdtuples[job["id"]]))
+        if self.pending_count[job["id"]] + len(self.job_stdtuples[job["id"]]) == 0:
+          progress = 0.0
+        else:
+          progress =  (1.0/10) * len(self.job_stdtuples[job["id"]]) / (self.pending_count[job["id"]] + len(self.job_stdtuples[job["id"]]))
         
         if self.pending_count[job["id"]] == 0:
             self.write_issues(self.job_issues[job["id"]]) 
@@ -413,6 +432,7 @@ class Pipeline(worker.Worker):
           return
         self.job_precalinds[job["id"]] = indicators_to_calculate
         self.job_precalstep[job["id"]] = 0
+        self.job_precalstamp[job["id"]] = time.time()
         self.update_job_step(job, 'precalculate_ended', 0.75)
 
     def send_to_calculate(self, indicators_to_calculate, job): 
@@ -426,14 +446,17 @@ class Pipeline(worker.Worker):
 
     def send_to_publish(self, tuples, job):
         calc_step = self.job_precalstep[job['id']]
-
-        self._variables_proxy.write_variable(tuples, calc_step, job)
+        calc_stamp = self.job_precalstamp[job['id']] 
+        self._variables_proxy.write_variable(tuples, calc_step, calc_stamp, job)
     
     def publish_end(self, job): 
         if job["status"] == "failed":
           return
         self.pending_count[job["id"]] = self.pending_count[job["id"]] - 1
-        progress =  ((self.pending_total[job["id"]] - self.pending_count[job["id"]]) / self.pending_total[job["id"]])/10
+        if self.pending_total[job["id"]] == 0:
+          progress == 0
+        else:
+          progress =  ((self.pending_total[job["id"]] - self.pending_count[job["id"]]) / self.pending_total[job["id"]])/10
         if self.pending_count[job["id"]] == 0:
           self.update_job_step(job, 'publish_ended', 1)
         else:
@@ -441,10 +464,10 @@ class Pipeline(worker.Worker):
     
     # this function returns a future that can be waited to ensure that file job is written to disk
     def update_job_step(self, job, step, progress):
-        l.debug(f"Changing to step {step} for job {int(job['id'])} source {job['dls_json']['scope']['source']} {round(progress*100, 1)}%")
+        l.info(f"Changing to step {step} for job {int(job['id'])} source {job['dls_json']['scope']['source']} {round(progress*100, 1)}%")
         #printMem()
         # removing job from current step dict
-        if job["step"] in self.job_steps and job["id"]:
+        if job["step"] in self.job_steps and job["id"] is not None:
           if job["id"] in self.job_steps[job["step"]]:
             self.job_steps[job["step"]].pop(job["id"])
           else:
@@ -462,7 +485,8 @@ class Pipeline(worker.Worker):
         else:
           self.job_steps[step][job["id"]] = job
 
-        return self._storage_proxy.write_db(job, 'job')
+        ret  = self._storage_proxy.write_db(job, 'job')
+        return ret
     
     # this function returns a future that can be waited to ensure that file job is written to disk
     def fail_job(self, job, delete_job = False, issue = None):
