@@ -69,8 +69,12 @@ class Storage(worker.Worker):
 
              
     def write_file(self, path, bytes, mode): #absolute path here
-        with open(path, mode) as f:
+       fd, fn = os.path.split(path)
+       tpath = os.path.join(fd, f'.{fn}.tmp')
+       with open(tpath, mode) as f:
             f.write(bytes) 
+       os.rename(tpath, path)
+
     
 
     def read_file(self, path): #absolute path here
@@ -161,16 +165,14 @@ class Storage(worker.Worker):
             if df.shape[0] > 0:
                 record['id'] = df.index.max()+1
             else:
-                record['id'] = 1
-            df.at[int(record['id'])] = record
+                record['id'] = 0
+            df = pd.concat([df, pd.DataFrame([record])], ignore_index = True)
         else:
             for key, value in record.items():
                 df.at[int(record['id']), key] = value
         self.db_tables[db_class] = df
         dest = os.path.join(os.getenv('PANDEM_HOME'), 'database', db_class+'s'+'.pickle') 
-        tmp = f"{dest}.tmp"
-        df.to_pickle(tmp)
-        shutil.move(tmp, dest)
+        util.save_pickle_df(df, dest)
         return record['id']
 
 
@@ -191,7 +193,8 @@ class Storage(worker.Worker):
             if len(to_del)>0:
               df = df.drop(index = to_del.index)
         self.db_tables[db_class] = df
-        df.to_pickle(os.path.join(os.getenv('PANDEM_HOME'), 'database', db_class+'s'+'.pickle'))
+        dest = os.path.join(os.getenv('PANDEM_HOME'), 'database', db_class+'s'+'.pickle')
+        util.save_pickle_df(df, dest)
         return df
 
     def get_job_cache(self, job_id):
@@ -215,6 +218,9 @@ class Storage(worker.Worker):
       return CacheValue(job_id, key, self._self_proxy)
 
     def from_job_cache(self, job_id, key):
+      return CacheValue(job_id, key, self._self_proxy)
+      
+    def value_from_job_cache(self, job_id, key):
       job_id = int(job_id)
       key = str(key)
       with self.get_job_cache(job_id) as cache:
@@ -232,7 +238,7 @@ class Storage(worker.Worker):
       if os.path.exists(shelve_path):
         os.remove(shelve_path)
 
-   
+       
  
 class CacheValue:
   def __init__(self, job_id, key, storage_proxy):
@@ -241,6 +247,6 @@ class CacheValue:
     self._storage_proxy = storage_proxy
 
   def value(self):
-    return self._storage_proxy.from_job_cache(self.job_id, self.key).get()
+    return self._storage_proxy.value_from_job_cache(self.job_id, self.key).get()
 
 
