@@ -1,5 +1,6 @@
 import os
 import datetime
+import pickle
 import json
 import numpy
 import io
@@ -7,6 +8,8 @@ import getpass
 import yaml
 import psutil
 import logging
+import hashlib
+import itertools
 l = logging.getLogger("pandem.perf")
 
 
@@ -72,8 +75,15 @@ def get_custom(path, function):
       return None
     else:
       return eval(f"{'.'.join(path)}.{function}")
+ 
+def md5(fname):
+    hash_md5 = hashlib.md5()
+    with open(fname, "rb") as f:
+        for chunk in iter(lambda: f.read(4096), b""):
+            hash_md5.update(chunk)
+    return hash_md5.hexdigest()
 
-
+ 
 class JsonEncoder(json.JSONEncoder):
   def default(self, z):
     if isinstance(z, datetime.datetime) or isinstance(z, datetime.date):
@@ -83,6 +93,33 @@ class JsonEncoder(json.JSONEncoder):
     else:
       return super().default(z)
 
+def slices(iterable, size):
+   head = list(itertools.islice(iterable, size))
+   while len(head) > 0:
+     yield head
+     head = list(itertools.islice(iterable, size))
+
+def save_pickle(o, path):
+  fd, fn = os.path.split(path)
+  tpath = os.path.join(fd, f'.{fn}.tmp')
+  with open(tpath, "wb") as f:
+    pickle.dump(o, f)
+  os.rename(tpath, path)
+
+def save_pickle_df(df, path):
+  fd, fn = os.path.split(path)
+  tpath = os.path.join(fd, f'.{fn}.tmp')
+  df.to_pickle(tpath)
+  os.rename(tpath, path)
+
+def save_json(o, path, indent = None):
+  fd, fn = os.path.split(path)
+  tpath = os.path.join(fd, f'.{fn}.tmp')
+  with open(tpath, 'w') as f:
+    json.dump(o, f, cls=JsonEncoder, indent = indent)
+  os.rename(tpath, path)
+
+
 def compress(x, get_id = {}, get_val = {}, i = [0]):
   if type(x) == dict:
     items = [*x.items()]
